/******************************************************************************************
 *	Chili DirectX Framework Version 12.10.21											  *
 *	Game.cpp																			  *
 *	Copyright 2012 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Game.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <float.h>
#include "Surface.h"

Game::Game( HWND hWnd/*,KeyboardServer& kServer,const MouseServer& mServer*/ )
:	gfx( hWnd ),
	audio( hWnd ),
	kbd(  ),
	mouse(  ),
	GamePad( hWnd ),
	playerScore(0),
	gState( Title ),
	eSpawn(0),
	aSpawn(0),
	p1ship(SCREENWIDTH,SCREENHEIGHT ),
	NAst( 0 ),
	mainmenu( "MainMenu" ),
	scoreboard( "ScoreBoard" ),
	CursorX( 100 ),
	CursorY( 100 ),
	CLF( false ),
	lIndex( 1 ),
	currentEnemyCount( 0 ),
	eSpawnRate( 0.5f ),
	aSpawnRate( 2.0f ),
	hWnd(hWnd),
	boss1( SCREENWIDTH,SCREENHEIGHT ),
	isAtBoss(false)
{
	// Init random generator
	srand( (unsigned int)time( NULL ) );

	// Load font sheet
	gfx.LoadFont( &f, sf, L"Fonts\\Fixedsys16x28.bmp", 16, 28, 32 );

	// Init menu objects
	/*scoreboard.AddScore( "Fred", scoreboard.GetTime(), 15 );
	scoreboard.AddScore( "George", scoreboard.GetTime(), 50 );
	scoreboard.AddScore( "Alex", scoreboard.GetTime(), 3 );
	scoreboard.AddScore( "Luis", scoreboard.GetTime(), 42 );
	scoreboard.AddScore( "Josh", scoreboard.GetTime(), 43 );*/
	scoreboard.SaveScoreList();
	Start = mainmenu.GetButton( mainmenu.Start );
	Score = mainmenu.GetButton( mainmenu.Score );
	Quit = mainmenu.GetButton( mainmenu.Quit );		
	Back = scoreboard.GetButton( scoreboard.Back );

	playerColor =	gfx.FILLCOLOR(255,   0, 255, 255); // Should be Cyan
	e1Color =		gfx.FILLCOLOR(255, 255, 128, 128); // Should be Redish
	ammoColor =		gfx.FILLCOLOR(255, 255, 255, 255); // Should be White
	astroColor =	gfx.FILLCOLOR(255, 128,  64,  32); // Should be Dark Orange
	
	// Load level 1
	LoadLevel( );
	camSpeed = pic.height / 120;
	
	// Start timers		
	AstTimer.StartWatch();
}

Game::~Game()
{
	
}

void Game::Go()
{
	DoState(gState);
	ComposeFrame();
	gfx.EndFrame();
}

void Game::LoadLevel( )
{
	// Load the level background image
	D3DGraphics::Color key = gfx.FILLCOLOR(0, 0, 0, 0);	
	level.LoadLevel( lIndex );
	LPCTSTR bgImg = level.GetBGImg();	
	gfx.LoadSpriteAlpha(&pic, bgImg, key);

	// Set camera position
	camera.vx = SCREENWIDTH / 2;
	camera.vy = pic.height - (SCREENHEIGHT / 2) -1;
	/*camera.vx = 399;
	camera.vy = 400;*/

	// Set player stats
	float temp = 1.0f - p1ship.GetHealth();
	p1ship.SetPosition( camera.vx, camera.vy );
	p1ship.SetHealth( -temp );
	// p1ship.SetClip(SCREENWIDTH, SCREENHEIGHT, camera.vx, camera.vy);
	temp = 1.0f - p1ship.GetShieldStrength();
	p1ship.SetShieldStrength( -temp );
	p1ship.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);

	Level::Trigger bTrigger = level.GetTrigger(level.GetNumEnemyTriggers() + 1);
	boss1.SetPosition(bTrigger.x, bTrigger.y);
}

bool Game::SwitchScreenMode()
{
HWND hWnd = gfx.GetWindowHWND();	// Window handle
bool IsWindowed = gfx.CheckWindowed();											// Flag windowed/fullscreen mode
long SavedWindowedWidth;// Saved windowed width
long SavedWindowedHeight;       // Saved windowed height
HMENU hMenu;                    // Handle to store menu between toggles
RECT ClientRect;                // Saved client rectangle
RECT WindowRect;                // Windowed windows rectangle
long FullscreenWidth;           // Fullscreen width
long FullscreenHeight;          // Fullscreen height
D3DPRESENT_PARAMETERS Params;   // Presentation Parameters
LPDIRECT3DDEVICE9 Device;       // D3D device
long WindowStyle;               // Windowed window style

// Make sure you pause any rendering
    // Are we in windowed mode at the moment
    if(IsWindowed)
    {
        // Set the fullscreen mode style, clear menu if attached
		SetWindowLong(hWnd, GWL_STYLE, WS_POPUP|WS_SYSMENU|WS_VISIBLE);
        if(hMenu == NULL)
        {
            hMenu = GetMenu(hWnd);
            SetMenu(hWnd, NULL);
        }

        // Save the windowed client dimensions
        SavedWindowedWidth = ClientRect.right - ClientRect.left;
        SavedWindowedHeight = ClientRect.bottom - ClientRect.top;

        // Set the Presentation Parameters, specifically
        Params.Windowed = FALSE;
        Params.BackBufferWidth = FullscreenWidth;
        Params.BackBufferHeight = FullscreenHeight;

        // Other params ... upto you!!!

        // Reset D3D device, any device dependent objects
        // that aren't managed or in system memory have
        // to pe released before you call Reset and recreated
        // after the reset, ditto for going back to windowed
        // mode. You also have to reset render states and matrices
        // after the call to Reset.
        if(FAILED(Device->Reset(&Params)))
        {
            // Couldn't do the change, set things back
            SetWindowLong(hWnd, GWL_STYLE, WindowStyle);
            if(hMenu != NULL)
            {
                SetMenu(hWnd, hMenu);
                hMenu = NULL;
            }

            // Set the window position
            SetWindowPos(hWnd, HWND_NOTOPMOST,
                         WindowRect.left, WindowRect.top,
                         (WindowRect.right - WindowRect.left),
                         (WindowRect.bottom - WindowRect.top),
                         SWP_SHOWWINDOW);

            return false;
        }

        IsWindowed = false;
    }
    // We're in fullscreen mode at the moment
    else
    {
        // Set the windowed mode style, reset menu if needed
        SetWindowLong(hWnd, GWL_STYLE, WindowStyle);
        if(hMenu != NULL)
        {
            SetMenu(hWnd, hMenu);
            hMenu = NULL;
        }

        // Set the Presentation Parameters, specifically
        Params.Windowed = TRUE;
        Params.BackBufferWidth = SavedWindowedWidth;
        Params.BackBufferHeight = SavedWindowedHeight;

        // Other params ... upto you!!!

        // Reset D3D device
        if(FAILED(Device->Reset(&Params)))
            return false;

        // Set the window position
        SetWindowPos(hWnd, HWND_NOTOPMOST,
                     WindowRect.left, WindowRect.top,
                     (WindowRect.right - WindowRect.left),
                     (WindowRect.bottom - WindowRect.top),
                     SWP_SHOWWINDOW);

        IsWindowed = true;
    }

    // Get new client rectangle
    GetClientRect(hWnd, &ClientRect);

    return true;
// Unpause and start rendering
}

void Game::DoState( Game::GameState State )
{	
	frame = GameTimer.GetTimeMilli();
	GameTimer.StopWatch();
	GameTimer.StartWatch();

	GetInput(&gInput);

	// Switch/case block to control the game
	switch (State)
	{
	case Title:
		MMBIC(&gInput);
		break;

	case ScoreBoard:
		SBBIC(&gInput);				
		break;

	case Play:
		UpdateCam();
		CheckForTriggers();		
		p1ship.DoState(gInput.x, gInput.y, fire);
		for(unsigned int i = 0; i < Enemies.size(); i++)
		{
			Enemies[ i ].DoState();
		}
		if(isAtBoss)
		{
			boss1.DoState( p1ship.GetShipCenterX(), p1ship.GetShipCenterY() );				
		}
		for( unsigned int index = 0; index < NAst; index++ )
		{
			Asteroids[ index ].Transition();
		}
		DetectCollision();
		Kill();
		break;

	case Pause:
		
		break;
	}
		
}

float Game::CalculateSpeed()
{
	return camSpeed * frame / 1000;
}

void Game::UpdateCam()
{
	if(camera.vy > SCREENHEIGHT / 2 + 1)
	{
		float camDeltaSpeed = min( CalculateSpeed(), 0.5f );
		camera.vy -= camDeltaSpeed;
		
		p1ship.SetClip(SCREENWIDTH, SCREENHEIGHT, camera.vx, camera.vy);
		for(unsigned int i = 0; i < Enemies.size(); i++)
		{
			Enemies[ i ].AdjustWaypoints( camDeltaSpeed );
		}
	}
}

void Game::CheckForTriggers()
{
	Level::TriggerType tType = level.IsAtTrigger( camera.vx, camera.vy - (SCREENHEIGHT / 2) ) ;
	if(tType == Level::ENEMY)
	{
		Spawn();
	}
	if(tType == Level::ASTEROID)
	{
		SpawnAsteroid();
	}
	if(tType == Level::BOSS)
	{
		isAtBoss = true;
	}
}

void Game::AsteroidCollision()
{
	Ship::Vertices* verts = {0};
	float pAmmoDamage = 0;
	float astroDamage = 0;
	unsigned int ammoListSize = p1ship.GetAmmoListSize();
	if(ammoListSize > 0)
	{
		verts = new Ship::Vertices[ ammoListSize ];
		pAmmoDamage = p1ship.GetDamage( p1ship.AMMO_DAMAGE );
	}
	float pShipDamage = p1ship.GetDamage( p1ship.SHIP_DAMAGE );
	float pShield = p1ship.GetShieldStrength();
	float pHealth = p1ship.GetHealth();

// Asteroids check against Player
	for(unsigned int i = 0; i < Asteroids.size(); i++)
	{
		OBJECT src1 = {Asteroids[ i ].GetX(), Asteroids[ i ].GetY(), Asteroids[ i ].GetRadius()};
		OBJECT src2 = {p1ship.GetShipCenterX(), p1ship.GetShipCenterY(), p1ship.GetShipSize() / 2};

		if( HasCollided(&src1, &src2) )
		{
			p1ship.SetHealth(pHealth);
			p1ship.SetShieldStrength( pShield );
			Asteroids[i].UpdateHealth( pShipDamage );
			Asteroids[ i ].SetHitByAmmo( false );
		}

// Asteroids check against Player Ammo
		for(unsigned int j = 0; j < ammoListSize; j++)
		{
			src2.x = verts->vx;
			src2.y = verts->vy;
			src2.radius = 2;

			if(HasCollided(&src1, &src2 ) )
			{
				p1ship.KillAmmo( j );
				Asteroids[ i ].UpdateHealth( pAmmoDamage );
				ammoListSize = p1ship.GetAmmoListSize();
				j--;
				Asteroids[ i ].SetHitByAmmo( true );
			}
		}

// Asteroids check against Asteroids
		for(unsigned int j = i + 1; j < Asteroids.size(); j++)
		{
			src2.x = Asteroids[ j ].GetX();
			src2.y = Asteroids[ j ].GetY();
			src2.radius = Asteroids[ j ].GetRadius();
			

			if( HasCollided( &src1, &src2) )
			{
				Asteroids[i].UpdateHealth( Asteroids[ j ].GetDamage() );
				Asteroids[j].UpdateHealth( Asteroids[ i ].GetDamage());
				Asteroids[ i ].SetHitByAmmo( false );
			}
		}
	}
	delete [] verts;
}

void Game::EnemyCollision()
{
	float shipDamage = 0;
	unsigned int eShipSize = 0;
	Ship::Vertices* verts = {0};
	float pAmmoDamage = 0;
	unsigned int ammoListSize = p1ship.GetAmmoListSize();
	if(ammoListSize > 0)
	{
		verts = new Ship::Vertices[ ammoListSize ];
		pAmmoDamage = p1ship.GetDamage( p1ship.AMMO_DAMAGE );
	}
	float pShipDamage = p1ship.GetDamage( p1ship.SHIP_DAMAGE );

//Enemies check against player
	for(unsigned int i = 0; i < Enemies.size(); i++)
	{
		eShipSize = Enemies[ i ].GetShipSize() / 2;
		OBJECT src1 = {Enemies[ i ].GetShipCenterX(), Enemies[ i ].GetShipCenterY(), eShipSize};
		OBJECT src2 = {p1ship.GetShipCenterX(), p1ship.GetShipCenterY(), p1ship.GetShipSize() / 2};		
		shipDamage = Enemies[ i ].GetDamage( Enemies[ i ].SHIP_DAMAGE );		

		if( HasCollided(&src1, &src2) )
		{
			Enemies[i].SetHealth( pShipDamage );
			p1ship.SetHealth( shipDamage );
			Enemies[ i ].SetHitByAmmo( false );
		}

// Enemies check against player ammo
		for(unsigned int j = 0; j < ammoListSize; j++)
		{
			verts[j] = p1ship.GetAmmoPos( j );		
			src2.x = verts[ j ].vx;
			src2.y = verts[ j ].vy;
			src2.radius = 2;
			if(HasCollided( &src1, &src2))
			{
				Enemies[ i ].SetHealth( pAmmoDamage );
				p1ship.KillAmmo(j);
				ammoListSize = p1ship.GetAmmoListSize();
				j--;
				Enemies[ i ].SetHitByAmmo( true );
			}
		}

// Enemies check against asteroids		
		float eHealth = Enemies[ i ].GetHealth();		
		
		for(unsigned int j = 0; j < Asteroids.size(); j++)
		{
			src2.x = Asteroids[ j ].GetX();
			src2.y = Asteroids[ j ].GetY();
			src2.radius = Asteroids[ j ].GetRadius();

			if( HasCollided( &src1, &src2) )
			{
				Enemies[ i ].SetHealth( eHealth );
				Asteroids[ j ].UpdateHealth( shipDamage );	
				Asteroids[ j ].SetHitByAmmo( false );
				Enemies[ i ].SetHitByAmmo( false );
			}
		}		
	}
	delete [] verts;
}

void Game::DetectCollision( )
{
	for(unsigned int i = 0; i < NAst; i++)
	{
		if(Asteroids[ i ].GetX() <  camera.vx - Asteroids[ i ].GetRadius() - (SCREENWIDTH  / 2) && Asteroids[ i ].GetXVol() < 0 ||
			Asteroids[ i ].GetX() >= camera.vx + Asteroids[ i ].GetRadius() + (SCREENWIDTH  / 2) && Asteroids[ i ].GetXVol() > 0 ||
			Asteroids[ i ].GetY() <  camera.vy - Asteroids[ i ].GetRadius() - (SCREENHEIGHT / 2) && Asteroids[ i ].GetYVol() < 0 ||
			Asteroids[ i ].GetY() >= camera.vy + Asteroids[ i ].GetRadius() + (SCREENHEIGHT / 2) && Asteroids[ i ].GetYVol() > 0 )
		{
			RemoveAsteroid( i );
			i--;
		}
	}

	if(isAtBoss)
	{
		B1Collison();
		EnemyCollision();
	}
	else
	{
		EnemyCollision();
		AsteroidCollision();
	}
}

// Menu functions
void Game::MMBIC(Input *gInput)
{
	CursorX = gInput->mX;
	CursorY = gInput->mY;
	if( CursorX > Start->X && CursorX < Start->X2 &&
		CursorY > Start->Y && CursorY < Start->Y2 )
	{
		CheckForLeftClick(gInput);
		mainmenu.SetButtonState( LeftIsPressed, true, GameMenu::Start );
		mainmenu.SetButtonColor( 0, 255, 255, GameMenu::Start );

		if( Start->isPressed )
		{
			mainmenu.SetButtonState( false, false, MainMenu::Start );
			gState = Play;
		}
	}
	else if( CursorX > Score->X && CursorX < Score->X2 &&
			 CursorY > Score->Y && CursorY < Score->Y2 )
	{
		CheckForLeftClick(gInput);
		mainmenu.SetButtonState( LeftIsPressed, true, GameMenu::Score );
		mainmenu.SetButtonColor( 0, 255, 255, GameMenu::Score );

		if( Score->isPressed )
		{
			mainmenu.SetButtonState( false, false, MainMenu::Score );
			gState = ScoreBoard;
		}
	}
	else if( CursorX > Quit->X && CursorX < Quit->X2 &&
			 CursorY > Quit->Y && CursorY < Quit->Y2 )
	{
		CheckForLeftClick(gInput);
		mainmenu.SetButtonState( LeftIsPressed, true, GameMenu::Quit );
		mainmenu.SetButtonColor( 0, 255, 255, GameMenu::Quit );
		
		if( Quit->isPressed )
		{
			mainmenu.SetButtonState( false, false, MainMenu::Quit );
			PostQuitMessage( 0 );
		}
	}
	else 
	{
		mainmenu.SetButtonState( false, false, GameMenu::Start );
		mainmenu.SetButtonColor( 255, 255, 255, GameMenu::Start );
		mainmenu.SetButtonState( false, false, GameMenu::Score );
		mainmenu.SetButtonColor( 255, 255, 255, GameMenu::Score );
		mainmenu.SetButtonState( false, false, GameMenu::Quit );
		mainmenu.SetButtonColor( 255, 255, 255, GameMenu::Quit );
	}
}

void Game::SBBIC(Input *gInput)
{
	CursorX = gInput->mX;
	CursorY = gInput->mY;

	if( CursorX > Back->X && CursorX < Back->X2 &&
		CursorY > Back->Y && CursorY < Back->Y2 )
	{
		CheckForLeftClick(gInput);
		scoreboard.SetButtonState( LeftIsPressed, true, GameMenu::Back );
		scoreboard.SetButtonColor( 0, 255, 255, GameMenu::Back );

		if( Back->isPressed )
		{
			scoreboard.SetButtonState( false, false, MainMenu::Back );
			gState = Title;
		}
	}
	else 
	{
		scoreboard.SetButtonState( false, false, GameMenu::Back );
		scoreboard.SetButtonColor( 255, 255, 255, GameMenu::Back );
	}
}

// Asteroid Functions
void Game::CreateAsteroid(Asteroid::Size aSize)
{
	Asteroid asteroid;

	if(aSize == Asteroid::Big)
	{
		Asteroids.push_back( asteroid );
		NAst++;	
		Level::Trigger aTrigger = level.GetTrigger( level.GetNumEnemyTriggers() );
		Asteroids[ NAst - 1 ].Init(aTrigger.x, aTrigger.y);
	}
	else
	{
		float X = Asteroids[ NAst - 1 ].GetX();
		float Y = Asteroids[ NAst - 1 ].GetY();
		unsigned int Radius = Asteroids[ NAst - 1].GetRadius();

		Asteroids.push_back( asteroid );
		NAst++;	
		Asteroids[ NAst - 1 ].SetSAstStats( X - Radius, Y - Radius );
		NAst++;	
		Asteroids[ NAst - 1 ].SetSAstStats( X + Radius, Y + Radius );
	}
}

void Game::RemoveAsteroid( unsigned int index )
{
	Asteroids.erase( Asteroids.begin() + index );
	NAst = Asteroids.size();
}

void Game::AsteroidBOD()
{
	for( int index = 0; index < Asteroids.size(); index++ )
	{
		Asteroid::Size size = Asteroids[ index ].GetSize();
		float health = Asteroids[ index ].GetHealth();

		if( health <= 0  )
		{
			if( size == Asteroid::Big )
			{
				if( Asteroids[ index ].GetHitByAmmo() )
				{
					AddScore( Asteroids[ index ].GetValue() );
				}
				CreateAsteroid(Asteroid::Small);				
			}
			else
			{
				if( Asteroids[ index ].GetHitByAmmo() )
				{
					AddScore( Asteroids[ index ].GetValue() );
				}
			}

			RemoveAsteroid( index );
		}		
	}
}

void Game::SpawnAsteroid()
{	
	aSpawn += frame;
	if(aSpawn >= aSpawnRate * 1000 )
	{
		aSpawn = 0;
		if( NAst < 10 )
		{
			level.ClearTrigger( false );
			CreateAsteroid(Asteroid::Big);
		}
		else
		{
			level.ClearTrigger( true );
		}
	}
}

// Helper functions
float Game::GetDistance(OBJECT* src1, OBJECT* src2)
{
	return sqrt( ( (src1->x - src2->x) * (src1->x - src2->x) ) + ( (src1->y - src2->y) * (src1->y - src2->y) ) );
}

bool Game::HasCollided(OBJECT* src1, OBJECT* src2)
{
	float Dist = GetDistance( src1, src2);
	return Dist <= src1->radius + src2->radius;
}

void Game::AddScore(DWORD shipValue)
{
	playerScore += shipValue;
}

void Game::CheckForLeftClick(Input *gInput )
{
		if( !CLF && gInput->LMB )
		{
			LeftIsPressed = false;
			CLF = true;
		}
		else if( CLF && !gInput->LMB )
		{
			LeftIsPressed = true;
			CLF = false;
		}
		else
		{
			LeftIsPressed = false;
			CLF = false;
		}
}

void Game::Spawn()
{
	unsigned int index = level.GetCurrentTrigger() - 1;
	Level::Trigger eTrigger = level.GetTrigger(index);
	float offSet = eTrigger.y - camera.vy;
	eSpawn += frame;
	if( eSpawn >= eSpawnRate * 1000 )
	{
		eSpawn = 0;
		if( currentEnemyCount < 5 && !eTrigger.passed )
		{	
			Enemy1 enemy1(SCREENWIDTH, SCREENHEIGHT);
			enemy1.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);
			enemy1.SetSpawnPoint( eTrigger.x, eTrigger.y - offSet - (SCREENHEIGHT / 2) );
			Enemies.push_back(std::move(enemy1));
			Enemy2 enemy2(SCREENWIDTH, SCREENHEIGHT);
			enemy2.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);
			enemy2.SetSpawnPoint( eTrigger.x, eTrigger.y  );
			Enemies.push_back(std::move(enemy2));
			Enemy3 enemy3(SCREENWIDTH, SCREENHEIGHT);
			enemy3.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);
			enemy3.SetSpawnPoint( eTrigger.x, eTrigger.y  );
			Enemies.push_back(std::move(enemy3));
			Enemy4 enemy4(SCREENWIDTH, SCREENHEIGHT);
			enemy4.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);
			enemy4.SetSpawnPoint( eTrigger.x + 250, eTrigger.y );
			Enemies.push_back(std::move(enemy4));
			Enemy5 enemy5(SCREENWIDTH, SCREENHEIGHT);
			enemy5.SetScreenRes(SCREENWIDTH, SCREENHEIGHT);
			enemy5.SetSpawnPoint( eTrigger.x + 300, eTrigger.y );
			Enemies.push_back(std::move(enemy5));
			currentEnemyCount++;
		}
		else
		{
			level.ClearTrigger( true );
			currentEnemyCount = 0;
		}
	}
}

void Game::GetInput(Input *gInput)
{
	// Clear joystick info
	gInput->x = 0;
	gInput->y = 0;
	gInput->r = 0;
	gInput->z = 0;
	kbd.Poll();

	if(GamePad.CheckJoystick())
	{
		// Poll the joystick, get joystick state
		GamePad.Poll();

		// Get X and Y position for joystick and normalize to +- 1
		gInput->x = (GamePad.GetJoyX() - 32767) / 32767;
		gInput->y = (GamePad.GetJoyY() - 32767) / 32767;
		gInput->r = (GamePad.GetJoyR() - 32767) / 32767;
		gInput->z = (GamePad.GetJoyZ() - 32767) / 32767;
		gInput->pButtons = GamePad.GetJoyButton();
		if(gInput->pButtons[0])
		{
			fire = true;
		}
		else
		{
			fire = false;
		}
	
		// Set deadzone for joystick X and Y to +- deadZone
		RemoveDeadZone(&gInput->x, &gInput->y, &gInput->z, &gInput->r);
	}
	else
	{
		// In game controls
		// W or UpArrow
		if( kbd.KeyIsPressed( DIK_W ) || kbd.KeyIsPressed( DIK_UP ) )
		{
			gInput->y = -1.0f;
		}
		// S or DownArrow
		if( kbd.KeyIsPressed( DIK_S ) || kbd.KeyIsPressed( DIK_DOWN ) )
		{
			gInput->y = 1.0f;
		}
		// A or LeftArrow
		if( kbd.KeyIsPressed( DIK_A ) || kbd.KeyIsPressed( DIK_LEFT ) )
		{
			gInput->x = -1.0f;
		}
		// D or RightArrow
		if( kbd.KeyIsPressed( DIK_D ) || kbd.KeyIsPressed( DIK_RIGHT ))
		{
			gInput->x = 1.0f;
		}
		if( kbd.KeyIsPressed( DIK_SPACE ))
		{
			fire = true;
		}
		else
		{
			fire = false;
		}
	}

	// Menu controls
	gInput->up = kbd.KeyIsPressed( DIK_UP ) ;
	gInput->down = kbd.KeyIsPressed( DIK_DOWN);
	gInput->left = kbd.KeyIsPressed( DIK_LEFT );
	gInput->right = kbd.KeyIsPressed( DIK_RIGHT );

	mouse.Poll();		
	gInput->mX += mouse.GetX();
	gInput->mY += mouse.GetY();
	gInput->LMB = mouse.BtnIsPressed(DIMOFS_BUTTON0 );
	if(gInput->mX < 0)
	{
		gInput->mX = 0;
	}
	if(gInput->mX >= SCREENWIDTH)
	{
		gInput->mX = SCREENWIDTH - 1;
	}
	if(gInput->mY < 0)
	{
		gInput->mY = 0;
	}
	if(gInput->mY >= SCREENHEIGHT)
	{
		gInput->mY = SCREENHEIGHT - 1;
	}
}

void Game::RemoveDeadZone(float* JoyX, float* JoyY, float* JoyZ, float* JoyR)
{
	if(*JoyX < 0.05f && *JoyX > -0.05f)
	{
		*JoyX = 0.0f;
	}
	if(*JoyY < 0.05f && *JoyY > -0.05f)
	{
		*JoyY = 0.0f;
	}
	if(*JoyZ < 0.05f && *JoyZ > -0.05f)
	{
		*JoyZ = 0.0f;
	}
	if(*JoyR < 0.05f && *JoyR > -0.05f)
	{
		*JoyR = 0.0f;
	}
}

void Game::Kill()
{
	AsteroidBOD();
	for( unsigned int i = 0; i < Enemies.size(); i++)
	{
		if(Enemies[ i ].GetHealth() <= 0.0f)
		{
			if( Enemies[ i ].GetHitByAmmo() )
			{
				AddScore( Enemies[ i ].GetShipValue() );
			}
			Enemies.erase(Enemies.begin() + i);
		}
	}			
}

bool Game::IsInView( OBJECT* object )
{
	if( object->x - object->radius >  camera.vx + (SCREENWIDTH  / 2) ||
		object->x + object->radius <= camera.vx - (SCREENWIDTH  / 2) ||
		object->y - object->radius >  camera.vy + (SCREENHEIGHT / 2) ||
		object->y + object->radius <= camera.vy - (SCREENHEIGHT / 2) )
	{
		return false;
	}
	else
	{
		return true;
	}
}

void Game::GetOffSets(Vertices* offSets)
{
	offSets->vx = camera.vx - (SCREENWIDTH / 2);
	offSets->vy = camera.vy - (SCREENHEIGHT / 2);
}

void Game::DrawShip(Ship* ship, D3DGraphics::Color c)
{
	Vertices offsets = {0};
	GetOffSets(&offsets);

	unsigned int nVerts = ship->GetNumVerts();
	Ship::Vertices* verts = ship->GetVertList();
	OBJECT shipPos = {0};
	shipPos.x = ship->GetShipCenterX();
	shipPos.y = ship->GetShipCenterY();
	shipPos.radius = ship->GetShipSize() / 2;

	int shieldRad = ship->GetShieldRad();
	float shieldStrength = ship->GetShieldStrength();
		
	if(IsInView( &shipPos ))
	{
		for(int i = 0; i < nVerts; i++)
		{
			int j = (i+1) % nVerts;
			gfx.DrawShip(shipPos.x - offsets.vx, shipPos.y - offsets.vy, 
						 verts[i].vx, verts[i].vy, verts[j].vx, verts[j].vy, c);
		}
		if(shieldStrength > 0)
		{
			gfx.DrawShield(shipPos.x - offsets.vx, shipPos.y - offsets.vy, shieldRad, shieldStrength);
		}
	}
}

void Game::DrawAmmo()
{
	unsigned int alSize = p1ship.GetAmmoListSize();
	Vertices offsets = {0};
	GetOffSets(&offsets);
	
	alSize = p1ship.GetAmmoListSize();
	for(int i = 0; i < alSize; i++)
	{
		Ship::Vertices Ammo = p1ship.GetAmmoPos(i);
		OBJECT ammo = {Ammo.vx, Ammo.vy, 2};
		if(IsInView( &ammo ))
		{
			gfx.DrawAmmo(Ammo.vx - offsets.vx, Ammo.vy - offsets.vy);
		}
		else
		{
			p1ship.KillAmmo(i);
			alSize = p1ship.GetAmmoListSize();
			i--;
		}
	}
}

void Game::DrawAsteroids()
{	
	Vertices offsets = {0};
	GetOffSets(&offsets);
	for( int index = 0; index < NAst; index++)
	{
		OBJECT asteroids = { Asteroids[ index ].GetX(), Asteroids[ index ].GetY(), Asteroids[ index ].GetRadius() };
		if(IsInView( &asteroids ))
		{
			gfx.DrawDisc( asteroids.x - offsets.vx, asteroids.y - offsets.vy, asteroids.radius, astroColor);
		}
	}
}

void Game::DrawCursor()
{
	gfx.DrawCircle( gInput.mX, gInput.mY, 16, 255, 255, 255);
}

void Game::DisplayScoreBoard()
{
	gfx.DrawButton( Back->X, Back->Y, Back->X2, Back->Y2, Back->Text.c_str(), &f, Back->R, Back->B, Back->G );

	std::vector< ScoreMenu::ScoreList > scorelist = scoreboard.GetScoreList();
	unsigned int y = 25;
	for( int index = 0; index < scorelist.size() && index < 10; index++ )
	{
		gfx.DrawString( scorelist[ index ].Text.c_str(), 50, y, &f, gfx.FILLCOLOR_XRGB( 255,255,255 ) );
		y += 50;
	}
}

void Game::DrawMenu()
{
	gfx.DrawButton( Start->X, Start->Y, Start->X2, Start->Y2, Start->Text.c_str(), &f, Start->R, Start->B, Start->G );
	gfx.DrawButton( Score->X, Score->Y, Score->X2, Score->Y2, Score->Text.c_str(), &f, Score->R, Score->B, Score->G );
	gfx.DrawButton( Quit->X, Quit->Y, Quit->X2, Quit->Y2, Quit->Text.c_str(), &f, Quit->R, Quit->B, Quit->G );
}

void Game::B1Draw()
{
	if(isAtBoss)
	{
		DrawShip( &boss1, gfx.FILLCOLOR_XRGB( 255,255,255 ) );
		for( int i = 0; i < boss1.GetNMissiles(); i++ )
		{
			Ship::Vertices mPos = boss1.GetMissile( i );
			if(mPos.vx != 0)
			{
				switch( boss1.MissileExploded( i ) )
				{
				case false:
					gfx.DrawCircle( mPos.vx, mPos.vy, boss1.GetMissileRad(), 0,255,0 );
					break;
				case true:
					gfx.DrawCircle( mPos.vx, mPos.vy, boss1.GetMissileRad(), 255,0,0 );
					break;
				}
			}
		}
	}
}

void Game::ComposeFrame()
{
	if(gState != Play)
	{
		gfx.BeginFrame();
		DrawCursor();
	}
	if(gState == Title)
	{
		DrawMenu();
	}
	else if(gState == ScoreBoard)
	{
		DisplayScoreBoard();
	}
	else if(gState == Play)
	{
		gfx.DrawBGImg(camera.vx, camera.vy, &pic);
		DrawShip(&p1ship, playerColor);
		for( int i = 0; i < Enemies.size(); i++)
		{		
			DrawShip(&Enemies[i], e1Color);
		}
		DrawAmmo();
		DrawAsteroids();
		B1Draw();

		char buffer[ 20 ];
		sprintf( buffer, "%d", playerScore );
		gfx.DrawString( buffer, 649, 8, &f, gfx.FILLCOLOR_XRGB( 255,255,255 ) );
	}
}

void Game::B1Collison()
{
	OBJECT player = {p1ship.GetShipCenterX(), p1ship.GetShipCenterY(), p1ship.GetShipSize() / 2};
	OBJECT boss = {boss1.GetShipCenterX(), boss1.GetShipCenterY(), boss1.GetShipSize() / 2};

	// Ship on ship collision check
	if(HasCollided( &player, &boss) )
	{
		float sStrength = p1ship.GetShieldStrength();
		if(sStrength <= 0.0f)
		{
			p1ship.SetHealth( boss1.GetDamage(Ship::SHIP_DAMAGE) );
			boss1.SetHealth( p1ship.GetDamage(Ship::SHIP_DAMAGE) );
		}
		else
		{
			p1ship.SetShieldStrength( boss1.GetDamage(Ship::SHIP_DAMAGE) );
			boss1.SetShieldStrength( p1ship.GetDamage(Ship::SHIP_DAMAGE) );
		}
	}

	// Missile on player collision check
	for(unsigned int i = 0; i < boss1.GetNMissiles(); i++)
	{
		Ship::Vertices missile = boss1.GetMissile( i );
		float mRad = boss1.GetMissileRad();
		OBJECT m = {missile.vx, missile.vy, mRad};

		if(boss1.MissileExploded( i ))
		{
			if( HasCollided(&player, &m) )
			{
				float sStrength = p1ship.GetShieldStrength();
				if(sStrength <= 0.0f)
				{
					p1ship.SetHealth( boss1.GetMissileDamage() );
				}
				else
				{
					p1ship.SetShieldStrength( boss1.GetMissileDamage() );
				}				
			}
		}
	}
	
	// Player ammo on Boss collision check
	unsigned int aListSize = p1ship.GetAmmoListSize();
	for(unsigned int j = 0; j < aListSize; j++)
	{
		Ship::Vertices aList = p1ship.GetAmmoPos( j );
		OBJECT ammo = {aList.vx, aList.vy, 2};
		if(HasCollided( &boss, &ammo ))
		{
			p1ship.KillAmmo(j);
			aListSize = p1ship.GetAmmoListSize();
			j--;
			boss1.SetHealth( p1ship.GetDamage( Ship::AMMO_DAMAGE) );
		}
	}	


}

