/******************************************************************************************
 *	Chili DirectX Framework Version 12.10.21											  *
 *	Game.h																				  *
 *	Copyright 2012 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#pragma once

#include "D3DGraphics.h"
#include "Keyboard.h"
#include "Mouse.h"
#include "Sound.h"
#include "Timer.h"
#include "FrameTimer.h"
#include "Surface.h"
#include "Joystick.h"
#include "Ship.h"
#include <vector>
#include "Asteroid.h"
#include "Menu.h"
#include "Level.h"

class Game
{
public:
	Game( HWND hWnd/*,KeyboardServer& kServer,const MouseServer& mServer*/ );
	~Game();
	void Go();

	// Game states: ShowTitle, PlayGame, Pause
	enum GameState
	{
		Title, ScoreBoard, Play, Pause, Options
	};

	// OBJECT keeps position and size
	struct Vertices
	{
		float vx, vy;
	};

	struct OBJECT
	{
		float x, y, radius;
	};


	struct Input
	{
		// Joystick input
		bool hasJoystick;
		float x,y,r,z;
		float deadZone;
		BYTE* pButtons;

		// Keyboard input
		bool enter, up, down, left, right;		

		// These inputs are for the menu in game control handled separately
		// Mouse input
		float mX, mY;
		bool LMB;
	};

private:
	void ComposeFrame();
	/********************************/
	/*  User Functions              */
	void DoState( GameState gState );
	void UpdateCam();
	bool SwitchScreenMode();

	//input functions
	void GetInput(Input *gInput);
	void RemoveDeadZone(float* JoyX, float* JoyY, float* JoyZ, float* JoyR);

	//State functions
	void Spawn();
	void Kill();
	void AsteroidCollision();
	void EnemyCollision();
	void DetectCollision( );
	void CheckForTriggers();

	//Draw functions
	void DrawShip(Ship* ship, D3DGraphics::Color c);
	void DrawAmmo();
	void DrawShield();
	void DrawAsteroids();

	// Asteroid functions
	void AsteroidBOD();
	void CreateAsteroid(Asteroid::Size);
	void RemoveAsteroid( unsigned int index );
	void SpawnAsteroid();

	// Helper functions
	bool HasCollided( OBJECT* src1, OBJECT* src2);
	float GetDistance(OBJECT* src1, OBJECT* src2);
	void AddScore(DWORD shipValue);
	void LoadLevel( );
	void UnloadLevel( );
	bool IsInView( OBJECT* object );
	float CalculateSpeed();
	void GetOffSets(Vertices* offSets);
	void CheckForLeftClick(Input *gInput);

	// Menu functions
	void MMBIC(Input *gInput);
	void SBBIC(Input *gInput);
	void DisplayScoreBoard();
	void DrawMenu();
	void DrawScoreboard();
	void DrawCursor();

	// Boss Functions
	void B1Collison();
	void B1Draw();

	/********************************/
private:
	D3DGraphics gfx;
	//KeyboardClient kbd;
	//MouseClient mouse;
	DIKeyB kbd;
	DIMouse mouse;
	DSound audio;
	DIJoystick GamePad;
	/********************************/
	/*  User Variables              */
	GameState gState;

	D3DGraphics::Font f;
	D3DGraphics::Color sf[512*84];	

	Player p1ship;
	bool fire;

	Timer AstTimer;
	Timer GameTimer;
	float eSpawn, aSpawn;

	float eSpawnRate, aSpawnRate;
	std::vector<Enemy> Enemies;
	Boss1 boss1;
	bool isAtBoss;
	unsigned int NAst;
	std::vector<Asteroid> Asteroids;

	D3DGraphics::Color playerColor;
	D3DGraphics::Color e1Color;
	D3DGraphics::Color ammoColor;
	D3DGraphics::Color astroColor;

	//MenuStuff//
	MainMenu::Button *Start;
	MainMenu::Button *Score;
	MainMenu::Button *Quit;
	ScoreMenu::Button *Back;

	unsigned int CursorX;
	unsigned int CursorY;
	bool LeftIsPressed;
	bool CLF;

	MainMenu mainmenu;
	ScoreMenu scoreboard;

	unsigned int playerScore;

	Vertices camera;
	float camSpeed;

	// Level stuff
	Level level;
	unsigned int lIndex;
	D3DGraphics::Sprite pic;

	float frame;
	unsigned int currentEnemyCount;

	Input gInput;
	HWND hWnd;
	/********************************/
};
