#include "Joystick.h"
#include <Windows.h>

Joystick::Joystick()
{
	ZeroMemory( &JoyCaps, sizeof(JOYCAPS) );
	joyGetDevCaps( 0, &JoyCaps, sizeof( JoyCaps ) );
	
	unsigned int dwSize = sizeof(JoyInfo);
	ZeroMemory(&JoyInfo, sizeof(JOYINFO));
	JoyInfo.dwSize = dwSize;
	JoyInfo.dwFlags = JOY_RETURNX | JOY_RETURNY | JOY_RETURNR | JOY_RETURNZ | JOY_RETURNBUTTONS;


	// Calculates the center of the joysticks.
	centerX = JoyCaps.wXmax / 2;
	centerY = JoyCaps.wYmax / 2;
	centerZ = JoyCaps.wZmax / 2;
	centerR = JoyCaps.wRmax / 2;
}

Joystick::~Joystick()
{
}

bool Joystick::CheckJoystick()
{
	if(joyGetPosEx(0, &JoyInfo) == JOYERR_NOERROR)
	{
		return true;
	}
	else
	{
		return false;
	}
}

float Joystick::GetJoyX()
{
	// Returns a normalized value between -1.0f and 1.0f for X axis.
	joyGetPosEx(0, &JoyInfo);
	float joyX = (float)JoyInfo.dwXpos;
	joyX = joyX - centerX;
	joyX = joyX / centerX;
	return joyX;
}

float Joystick::GetJoyY()
{
	// Returns a normalized value between -1.0f and 1.0f for Y axis.
	joyGetPosEx(0, &JoyInfo);
	float joyY = (float)JoyInfo.dwYpos;
	joyY = joyY - centerY;
	joyY = joyY / centerY;
	return joyY;
}

float Joystick::GetJoyZ()
{
	// Returns a normalized value between -1.0f and 1.0f for Z axis.
	joyGetPosEx(0, &JoyInfo);
	float joyZ = (float)JoyInfo.dwZpos;
	joyZ = joyZ - centerZ;
	joyZ = joyZ / centerZ;
	return joyZ;
}

float Joystick::GetJoyR()
{
	// Returns a normalized value between -1.0f and 1.0f for R axis.
	joyGetPosEx(0, &JoyInfo);
	float joyR = (float)JoyInfo.dwRpos;
	joyR = joyR - centerR;
	joyR = joyR / centerR;
	return joyR;
}

unsigned int Joystick::GetJoyButton()
{
	// Returns which joystick button was pressed
	return JoyInfo.dwButtons;
}

BOOL CALLBACK DIEnumDevices( LPCDIDEVICEINSTANCE lpddi, LPVOID lpvRef )
{
	// some complex pointer stuff huh xD
	// i pretty much convert the dwords to pointers
	LPDIRECTINPUTDEVICE8* lpPad = (LPDIRECTINPUTDEVICE8*)((DWORD*)lpvRef)[1];
	((LPDIRECTINPUT8)(*(DWORD*)lpvRef))->CreateDevice( lpddi->guidInstance, lpPad, NULL );

	DIDEVCAPS diCaps = *(LPDIDEVCAPS)(((DWORD*)lpvRef)[2]);
	ZeroMemory( &diCaps, sizeof(DIDEVCAPS) );
	(*lpPad)->GetCapabilities( &diCaps );
	if( diCaps.dwAxes < 4 )
		return DIENUM_CONTINUE;
	
	return DIENUM_STOP;
}

DIJoystick::DIJoystick( HWND hwnd )
: lpDInput( NULL ), pPad( NULL )
{
	DirectInput8Create( GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&lpDInput, NULL );
	
	HRESULT hr = lpDInput->CreateDevice( GUID_Joystick, &pPad, NULL);
	
	if( !pPad )
	{
		return;
	}

	ZeroMemory( &diCaps, sizeof(DIDEVCAPS) );
	diCaps.dwSize = sizeof(DIDEVCAPS);
	hr = pPad->GetCapabilities( &diCaps );

	// set dead zone
	
	ZeroMemory( &dipdw, sizeof(dipdw) );
	dipdw.diph.dwSize = sizeof(dipdw);
	dipdw.diph.dwHeaderSize = sizeof(dipdw.diph);
	dipdw.diph.dwHow = DIPH_DEVICE;

	// set buffer size
	dipdw.dwData = sizeof(DIDEVICEOBJECTDATA);
	pPad->SetProperty( DIPROP_BUFFERSIZE, &dipdw.diph );

	//dipdw.dwData = 500; // max value is 10,000, so 500 is 5% of 10,000
	//pPad->SetProperty( DIPROP_DEADZONE, &dipdw.diph );	
	//
	//dipdw.dwData = DIPROPAUTOCENTER_ON;
	//pPad->SetProperty( DIPROP_AUTOCENTER, &dipdw.diph );
	//
	//dipdw.dwData = DIPROPAXISMODE_ABS;
	//pPad->SetProperty( DIPROP_AXISMODE, &dipdw.diph );
	
	pPad->SetDataFormat( &c_dfDIJoystick );
	pPad->SetCooperativeLevel( hwnd, DISCL_FOREGROUND|DISCL_EXCLUSIVE );
	pPad->Acquire();
	
	ZeroMemory( &diJState, sizeof(diJState) );
}

DIJoystick::~DIJoystick()
{
	if( pPad )
		pPad->Release();
	pPad = NULL;
	if( lpDInput )
		lpDInput->Release();
	lpDInput = NULL;
}

bool DIJoystick::CheckJoystick()
{
	return pPad != NULL;
}

void DIJoystick::Poll()
{
	pPad->Acquire();
	pPad->GetDeviceState(sizeof(diJState), &diJState );
	
}

float DIJoystick::GetJoyX()
{
	return (float)diJState.lX;
}

float DIJoystick::GetJoyY()
{
	return (float)diJState.lY;
}

float DIJoystick::GetJoyZ()
{
	return (float)diJState.rglSlider[0];
}

float DIJoystick::GetJoyR()
{
	return (float)diJState.lRz;
}

BYTE* DIJoystick::GetJoyButton()
{
	return diJState.rgbButtons;
}
