/******************************************************************************************
 *	Chili DirectX Framework Version 12.10.21											  *
 *	Keyboard.cpp																		  *
 *	Copyright 2012 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Keyboard.h"

KeyboardClient::KeyboardClient( const KeyboardServer& kServer )
	: server( kServer )
{}

bool KeyboardClient::KeyIsPressed( unsigned char keycode ) const
{
	return server.keystates[ keycode ];
}

unsigned char KeyboardClient::ReadKey()
{
	if( !server.keybuffer.empty() )
	{
		unsigned char keycode = server.keybuffer.front();
		const_cast<std::queue<unsigned char>&>(server.keybuffer).pop();
		return keycode;
	}
	return 0;
}

unsigned char KeyboardClient::PeekKey() const
{
	if( !server.keybuffer.empty() )
	{
		return server.keybuffer.front();
	}
	return 0;
}

unsigned char KeyboardClient::ReadChar()
{
	if( !server.charbuffer.empty() )
	{
		unsigned char charcode = server.charbuffer.front();
		const_cast<std::queue<unsigned char>&>(server.charbuffer).pop();
		return charcode;
	}
	return 0;
}

unsigned char KeyboardClient::PeekChar() const
{
	if( !server.charbuffer.empty() )
	{
		return server.charbuffer.front();
	}
	return 0;
}

void KeyboardClient::FlushKeyBuffer()
{
	while( !server.keybuffer.empty() )
	{
		const_cast<std::queue<unsigned char>&>(server.keybuffer).pop();
	}
}

void KeyboardClient::FlushCharBuffer()
{
	while( !server.charbuffer.empty() )
	{
		const_cast<std::queue<unsigned char>&>(server.charbuffer).pop();
	}
}

void KeyboardClient::FlushBuffers()
{
	FlushKeyBuffer();
	FlushCharBuffer();
}

KeyboardServer::KeyboardServer()
{
	for( int i = 0; i < nKeys; i++ )
	{
		keystates[ i ] = false;
	}
}

void KeyboardServer::OnKeyPressed( unsigned char keycode )
{
	keystates[ keycode ] = true;
	
	keybuffer.push( keycode );
	if( keybuffer.size() > bufferSize )
	{
		keybuffer.pop();
	}
}

void KeyboardServer::OnKeyReleased( unsigned char keycode )
{
	keystates[ keycode ] = false;
}

void KeyboardServer::OnChar( unsigned char character )
{
	charbuffer.push( character );
	if( charbuffer.size() > bufferSize )
	{
		charbuffer.pop();
	}
}

void KeyboardServer::ClearState()
{
	for( int i = 0; i < nKeys; i++ )
	{
		keystates[ i ] = false;
	}
}


DIKeyB::DIKeyB()
: pKeyB(NULL)
{
	ZeroMemory( keys, sizeof(BYTE)*256 );
	lpdi->CreateDevice( GUID_SysKeyboard, &pKeyB, NULL );
	pKeyB->SetDataFormat( &c_dfDIKeyboard );
	pKeyB->SetCooperativeLevel( hwnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE );
	
	DIPROPDWORD dipd;
	dipd.diph.dwSize = sizeof(DIPROPDWORD);
	dipd.diph.dwHeaderSize = sizeof(DIPROPHEADER);
	dipd.diph.dwObj = 0;
	dipd.diph.dwHow = DIPH_DEVICE;
	dipd.dwData = sizeof(DIDEVICEOBJECTDATA);
	pKeyB->SetProperty( DIPROP_BUFFERSIZE, &dipd.diph );
}

DIKeyB::~DIKeyB()
{
	pKeyB->Unacquire(), pKeyB->Release(), pKeyB = NULL;
}

void DIKeyB::Poll()
{
	pKeyB->Acquire();
	pKeyB->Poll();
	pKeyB->GetDeviceState( sizeof(BYTE)*256, keys );
}

// use DIK_XXX for key
bool DIKeyB::KeyIsPressed( unsigned char key )
{
	return keys[key] == 0x80;
}

char DIKeyB::ReadKey()
{
	DWORD dw = 1;
	DIDEVICEOBJECTDATA didod;
	pKeyB->GetDeviceData( sizeof(DIDEVICEOBJECTDATA), &didod, &dw, 0 );
	return dw ? didod.dwOfs : 0;
}

bool DIKeyB::PeekKey()
{
	DWORD dw = 1;
	pKeyB->GetDeviceData( sizeof(DIDEVICEOBJECTDATA), NULL, &dw, DIGDD_PEEK );
	return dw;
}
