#pragma once

#include <string>
#include <cstdio>
#include <vector>
#include <windows.h>
#include <time.h>
#include <fstream>
// #include <iostream>

class GameMenu
{
public:
	GameMenu( std::string fileName );
	~GameMenu();

	struct Button
	{
		std::string Text;
		unsigned int X, Y, X2, Y2;
		DWORD R, G, B;
		bool isSelected, isPressed;
	};

	enum ButtonType 
	{
		Score, Start, Quit, Back, Options, Resume
	};
	virtual void SetButtonState( bool isPressed, bool isSelected, ButtonType buttontype );
	virtual Button* GetButton( ButtonType buttontype );
	virtual void SetButtonColor( DWORD R, DWORD G, DWORD B, ButtonType buttontype );

public:
	std::string GetImageName();

protected:
	std::string fileName; // Background image
};

class MainMenu : public GameMenu
{
public:
	MainMenu( std::string fileName );
	~MainMenu();

	virtual void SetButtonState( bool isPressed, bool isSelected, ButtonType buttontype );
	virtual Button* GetButton( ButtonType buttontype );
	virtual void SetButtonColor( DWORD R, DWORD G, DWORD B, ButtonType buttontype );
	
private:
	Button mScoreButton;
	Button mStartButton;
	Button mQuitButton;
	Button mResume;
	Button mOptions;
};

class ScoreMenu : public GameMenu
{
public:
	ScoreMenu( std::string fileName );
	~ScoreMenu();

	struct ScoreList
	{
		std::string Name, Date;
		std::string Text;
		unsigned int Score;
	};

	virtual void SetButtonState( bool IsPressed, bool isSelected, ButtonType buttontype );
	virtual Button* GetButton( ButtonType buttontype );
	virtual void SetButtonColor( DWORD R, DWORD G, DWORD B, ButtonType buttontype );

	std::vector< ScoreList > GetScoreList();
	void AddScore( std::string Name, std::string Date, unsigned int Score );
	std::string GetTime();
	void SaveScoreList();

private:
	void LoadScoreList();
	void SortScoreList();

	ScoreList scorelist;
	std::vector< ScoreList > list;
	unsigned int LSize;
	Button mBackButton;
	FILE* pfile;

};

class OptionsMenu : public GameMenu
{
	OptionsMenu();
	~OptionsMenu();

	void LoadConfig();
	void SaveConfig();

public:
	
	void GetControlsOptions( char* controllerType, char** keyAssignments );	
	void GetGraphicsOptions( unsigned int* scrnRes, bool isFullScreen );
	void GetSoundOptions( float musicVolume, float sfxVolume );

private:
	enum key
	{
		UP, DOWN, LEFT, RIGHT, FIRE
	};
	char* controller;
	char* keyAssign[5];
	unsigned int resX, resY;
	bool fScreen;
	float mVolume, sfxVolume;
	char* fileName, header;
	Button mBackButton;

};