/******************************************************************************************
 *	Chili DirectX Framework Version 12.10.21											  *
 *	Mouse.cpp																			  *
 *	Copyright 2012 PlanetChili <http://www.planetchili.net>								  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Mouse.h"

MouseClient::MouseClient( const MouseServer& server )
: server( server )
{}
int MouseClient::GetX() const
{
	return server.x;
}
int MouseClient::GetY() const
{
	return server.y;
}
bool MouseClient::LeftIsPressed() const
{
	return server.leftIsPressed;
}
bool MouseClient::MiddleIsPressed() const
{
	return server.middleIsPressed;
}
bool MouseClient::RightIsPressed() const
{
	return server.rightIsPressed;
}
bool MouseClient::IsInWindow() const
{
	return server.isInWindow;
}

MouseServer::MouseServer()
:	isInWindow( false ),
	leftIsPressed( false ),
	rightIsPressed( false ),
	x( -1 ),
	y( -1 )
{}
void MouseServer::OnMove( int x,int y )
{
	this->x = x;
	this->y = y;
}
void MouseServer::OnLeave()
{
	isInWindow = false;
}
void MouseServer::OnEnter()
{
	isInWindow = true;
}
void MouseServer::OnLeftPressed()
{
	leftIsPressed = true;
}
void MouseServer::OnLeftReleased()
{
	leftIsPressed = false;
}
void MouseServer::OnMiddlePressed()
{
	middleIsPressed = true;
}
void MouseServer::OnMiddleReleased()
{
	middleIsPressed = false;
}
void MouseServer::OnRightPressed()
{
	rightIsPressed = true;
}
void MouseServer::OnRightReleased()
{
	rightIsPressed = false;
}
bool MouseServer::IsInWindow() const
{
	return isInWindow;
}


DIMouse::DIMouse()
: pMouse(NULL)
{
	ZeroMemory( &dims, sizeof(dims) );
	
	lpdi->CreateDevice( GUID_SysMouse, &pMouse, NULL );
	pMouse->SetDataFormat( &c_dfDIMouse2 );
	pMouse->SetCooperativeLevel( hwnd, DISCL_FOREGROUND | DISCL_EXCLUSIVE );
	
	DIPROPDWORD dipd;
	dipd.diph.dwSize = sizeof(DIPROPDWORD);
	dipd.diph.dwHeaderSize = sizeof(DIPROPHEADER);
	dipd.diph.dwObj = 0;
	dipd.diph.dwHow = DIPH_DEVICE;
	dipd.dwData = sizeof(DIDEVICEOBJECTDATA);
	pMouse->SetProperty( DIPROP_BUFFERSIZE, &dipd.diph );
}

DIMouse::~DIMouse()
{
	pMouse->Unacquire(), pMouse->Release(), pMouse = NULL;
}

void DIMouse::Poll()
{
	pMouse->Acquire();
	pMouse->Poll();
	pMouse->GetDeviceState( sizeof(DIMOUSESTATE2), &dims );
}

short DIMouse::GetX()
{
	return dims.lX;
}

short DIMouse::GetY()
{
	return dims.lY;
}

// Z in this case is the mouse wheel
short DIMouse::GetZ()
{
	return dims.lZ;
}

// use DIMOFS_XXX for btn
bool DIMouse::BtnIsPressed( BYTE btn )
{
	return dims.rgbButtons[0] == 0x80;
}

// Next 2 functions are for buffered data, they might be useful for something
char DIMouse::Read()
{
	DWORD dw = 1;
	DIDEVICEOBJECTDATA didod;
	pMouse->GetDeviceData( sizeof(DIDEVICEOBJECTDATA), &didod, &dw, 0 );
	return dw ? didod.dwOfs : 0;
}

bool DIMouse::Peek()
{
	DWORD dw = 1;
	pMouse->GetDeviceData( sizeof(DIDEVICEOBJECTDATA), NULL, &dw, DIGDD_PEEK );
	return dw;
}
