/******************************************************************************************
 *	Chili DirectX Framework Version 12.10.21											  *
 *	Mouse.h																				  *
 *	Copyright 2012 PlanetChili <http://www.planetchili.net>								  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#pragma once
#include "Input.h"

class MouseServer;

class MouseClient
{
public:
	MouseClient( const MouseServer& server );
	int GetX() const;
	int GetY() const;
	bool LeftIsPressed() const;
	bool MiddleIsPressed() const;
	bool RightIsPressed() const;
	bool IsInWindow() const;
private:
	const MouseServer& server;
};

class MouseServer
{
	friend MouseClient;
public:
	MouseServer();
	void OnMove( int x,int y );
	void OnLeave();
	void OnEnter();
	void OnLeftPressed();
	void OnLeftReleased();
	void OnMiddlePressed();
	void OnMiddleReleased();
	void OnRightPressed();
	void OnRightReleased();
	bool IsInWindow() const;
private:
	int x;
	int y;
	bool leftIsPressed;
	bool middleIsPressed;
	bool rightIsPressed;
	bool isInWindow;
};


class DIMouse : DInput
{
public:
	DIMouse();
	~DIMouse();
	void Poll();
	short GetX();
	short GetY();
	short GetZ();
	bool BtnIsPressed( BYTE );
	char Read();	// might be useful for something
	bool Peek();

private:
	IDirectInputDevice8 *pMouse;
	DIMOUSESTATE2 dims;
	DIPROPHEADER diProp;
};
