#include "Projectiles.h"

Projectile::Projectile()
{
}

Projectile::Projectile( int originX,  int originY)
	:
	TTL(2000),	
	speed(800),
	x(originX),
	y(originY)
{
	startMoveTime = startLiveTime = 0;
}

void Projectile::Move(bool isFromPlayer)
{
	stopMoveTime = timer.GetTimeMilli();
	float temp = stopMoveTime - startMoveTime;
	float a = speed / 1000 * temp;
	if(isFromPlayer)
	{
		y -= a;
	}
	else
	{
		y += a;
	}
	startMoveTime = stopMoveTime;

}

bool Projectile::HasTimeToLive()
{
	stopLiveTime = timer.GetTimeMilli();
	if(stopLiveTime < TTL)
	{
		startLiveTime = stopLiveTime;
		return true;
	}
	else
	{
		return false;
	}

}

Projectile::~Projectile(void)
{
	
}

// Missile Class Begin
Missile::Missile()
	:
	MissileD( 10 ),
	MIR( 10 ),
	ExplodeTimer(0),
	LaunchTimer(0),
	explodeDuration(0.5f)
{
	TTL = 1.0f;
	targetAcquired = false;
}

Missile::Missile(int originX, int originY) 
	: 
	Projectile(  originX,  originY),
	MissileD( 10 ),
	MIR( 10 ),
	explodeDuration(0.5f)
{
	TTL = 1.0f;
	targetAcquired = false;
}

void Missile::DecideState()
{
	frameTimer = mTimer.GetTimeMilli();
	mTimer.StopWatch();
	mTimer.StartWatch();

	if(!targetAcquired)
	{
		if(Fired)
		{
			mState = Acquire;
		}
		else
		{
			mState = None;
		}
	}
	else if(LaunchTimer < TTL * 1000 && targetAcquired)
	{
		mState = Launched;
	}
	else
	{
		if( ExplodeTimer < explodeDuration * 1000)
		{
			mState = Explode;
		}
		else
		{
			mState = Reset;
		}
	}
}

void Missile::DoState( int tX, int tY, bool canFire )
{
	Fired = canFire;
	DecideState();
	switch(mState)
	{
	case Acquire:
		AcquireTarget(tX, tY);
		break;
	case Launched:
		LaunchMissiles();
		break;
	case Explode:
		Exploded();
		break;
	case Reset:
		ResetMissile();					// break left out on purpose
	case None:
		break;
	}
}

void Missile::AcquireTarget(  int tX,  int tY )
{
	// Randomize X, Y +/- 100 from player position
	x = (rand() % 200 - 100) + tX;
	y = (rand() % 200 - 100) + tY;	
	targetAcquired = true;
}

void Missile::LaunchMissiles()
{
	LaunchTimer += frameTimer;
}

void Missile::Exploded()
{
	ExplodeTimer += frameTimer;
}

void Missile::ResetMissile( )
{
	x = 0;
	y = 0;
	ExplodeTimer = 0;
	LaunchTimer = 0;
	targetAcquired = false;
}

bool Missile::HasExploded()
{
	return mState == Explode;
}

int Missile::GetX() const
{
	return x + 0.5f;
}

int Missile::GetY() const
{
	return y + 0.5f;
}

int Missile::GetMIR() const
{
	return MIR;
}

const float Missile::GetDamage()
{
	return MissileD;
}
