#include "Ship.h"
/*  The Ship class is the parent class and will store all functions that are not
	specific to any one ship and it will store virtual functions that will be
	defined in child classes.  Any data that is ship specific will be listed in
	the child class while common data such as speed will be stored in the parent 
	class as protected with a Get function for retrieving the data.*/

/*  Ships can be created by listing the X and Y distance from the center of the ship
	in the constructor of the child class.
	If ships take up 32x32 then center would be x=15, y=15
	List the vertices either in a clockwise or anticlockwise manner, as long as each
	point connects, the DrawShip function in Game will connect the last point in the 
	list to the first point. */

Ship::Ship(int scrnWidth, int scrnHeight)	
{
}

float Ship::GetSpeed(float input)
{
	float temp = input / 1000;
	return Speed * temp;
}

unsigned int Ship::GetShieldRad()
{
	return shieldRad;
}

unsigned int Ship::GetNumVerts()const
{
	return numVerts;
}

unsigned int Ship::GetShipSize()
{
	return shipSize;
}

float Ship::GetShipCenterX()
{
	return shipCenterX;
}

float Ship::GetShipCenterY()
{
	return shipCenterY;
}

float Ship::GetShieldStrength()
{
	return shield.strength;
}

Ship::Vertices* Ship::GetVertList()
{
	return verts;
}

void Ship::KillAmmo(unsigned int ammoIndex)
{
	ammoVector.erase(ammoVector.begin() + ammoIndex);
}

void Ship::UpdateAmmo(unsigned int ammoListSize)
{
	for(int i = 0; i < ammoListSize; i++)
	{
		if(ammoVector[i].HasTimeToLive())
		{
			ammoVector[i].Move(true);
		}
		else
		{
			KillAmmo(i);
			if(ammoListSize != ammoVector.size())
			{
				ammoListSize = ammoVector.size();
			}
			i--;
		}
	}
}

Ship::Vertices Ship::GetAmmoPos(unsigned int ammoIndex)
{
	Vertices Ammo = { ammoVector[ammoIndex].x, ammoVector[ammoIndex].y };
	return Ammo;
}

unsigned int Ship::GetAmmoListSize()
{
	return ammoVector.size();
}

void Ship::SetShieldStrength(float damage)
{
	shield.strength -= damage;
}

float Ship::GetDamage( Ship::DamageType damage)
{	
	float sDamage = 0.0f;
	switch(damage)
	{
	case SHIP_DAMAGE:
		sDamage = shipDamage;
		break;
	case AMMO_DAMAGE:
		sDamage = ammoDamage;
		break;		
	}	
	return sDamage;
}

void Ship::SetHealth( float health )
{
	shipHealth -= health;
}

float Ship::GetHealth()
{
	return shipHealth;
}

unsigned int Ship::GetShipValue()
{
	return shipValue;
}

void Ship::SetHitByAmmo( bool wasHit )
{
	wasHitByAmmo = wasHit;
}

bool Ship::GetHitByAmmo()
{
	return wasHitByAmmo;
}

// Helper Functions
bool Ship::CheckIsAlive( )
{
	return shipHealth > 0.0f;
}

float Ship::CalcRun( float x, float x2 )
{
	return abs( x2 - x );
}

float Ship::CalcRise( float y, float y2 )
{
	return abs( y2 - y );
}

float Ship::CalcSlope( float rise, float run )
{
	if(rise > run)
	{
		return run / rise;
	}
	else if(rise < run)
	{
		return rise / run;
	}
	else
	{
		return 1.0f;
	}
}

void Ship::SetClip(unsigned int x, unsigned int y, unsigned int x2, unsigned int y2)
{
	scrnWidth = x;
	scrnHeight = y;
	camx = x2;
	camy = y2;
}

void Ship::SetPosition( unsigned int x, unsigned int y )
{
	shipCenterX = x;
	shipCenterY = y;
}

void Ship::SetScreenRes( unsigned int x, unsigned int y)
{
	scrnWidth = x;
	scrnHeight = y;
}

Ship::~Ship()
{
}

//  ------------End of Ship class-------------


Player::Player(int scrnWidth, int scrnHeight)
	: Ship(scrnWidth, scrnHeight)
{	
	BONUS = NONE;
	// Pixels per second
	Speed = 350;													
	numVerts = 4;
	shieldRad = 40;
	shipSize = 32;
	shipCenterX = 400;
	shipCenterY = 300;
	
	shipHealth = 1.0f;
	ammoDamage = 0.50f;
	shipDamage = 0.20f;

	shield.strength = 1.0f;
	shield.rechargeRate = 0.01f;
	shield.respawnMax = 0.8f;
	
	lives = 3;

	verts = new Vertices[numVerts];

	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;

	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 15;

	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 10;

	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 15;
}

void Player::Move( float scrnX, float scrnY)
{
	float moveTime = GetTime(MOVE_TIMER);
	ResetTimer( MOVE_TIMER );
	shipCenterX += min(5, GetSpeed(scrnX) * moveTime);
	shipCenterY += min(5, GetSpeed(scrnY) * moveTime);
	Clip(scrnWidth, scrnHeight, camx, camy);
}

void Player::Fire()
{
	Projectile ammo(shipCenterX, shipCenterY);
	ammoVector.push_back(ammo);
}

int Player::GetLives()
{
	return lives;
}

void Player::Die()
{
	if(lives > 0)
	{
		shield.strength = 1.0f;
		shipHealth = 1.0f;
		lives--;
		SetPosition(camx, camy);
	}
}

void Player::RechargeShield()
{
	if(shield.strength < 1.0f)
	{
		if(GetTime(RECHARGE_TIMER) > 1000)
		{					
			shield.strength += 0.01f;
			ResetTimer( RECHARGE_TIMER );
		}
	}
}

bool Player::CheckReadyToFire()
{	
	if( GetTime(FIRE_TIMER) >= 100 )
	{
		ResetTimer( FIRE_TIMER );
		return true;
	}
	return false;
}

void Player::DecideState( bool isShooting )
{
	bool isReady = CheckReadyToFire();
	if(shipHealth <= 0.0f)
	{
		state = Dead;
	}
	else if(isShooting )
	{
		if( isReady )
		{
			state = Firing;
		}
		else
		{
			state = Moving;
		}
	}
	else
	{
		state = Moving;
	}
}

void Player::DoState( float X, float Y, bool isFiring)
{	
	DecideState( isFiring );

	switch(state)
	{
	case Firing:
		Fire();								// No break; on purpose.  When player fires ship should also move same frame
	case Moving:
		Move( X, Y);
		UpdateAmmo(ammoVector.size());
		RechargeShield();
		break;
	case Dead:
		Die();
		break;
	}
}

void Player::Clip(int scrnWidth, int scrnHeight, int camx, int camy)
{
	int Radius = shipSize / 2;
	unsigned int left = camx - (scrnWidth / 2);
	unsigned int top = camy - (scrnHeight / 2);
	unsigned int right = camx + (scrnWidth / 2);
	unsigned int bottom = camy + (scrnHeight / 2);

	if(shipCenterX <=  left + Radius )
	{
		shipCenterX = left + Radius;
	}
	if(shipCenterY <= top + Radius )
	{
		shipCenterY = top + Radius;
	}
	if(shipCenterX > right - Radius)
	{
		shipCenterX = right - Radius;
	}
	if(shipCenterY > bottom - Radius)
	{
		shipCenterY = bottom - Radius;
	}	
}

void Player::AddBonus(float* adjSpeed, float* adjShield, int* adjLives)
{
	if(BONUS & SPEED)
	{
		*adjSpeed = Speed + 0.75f;
	}
	if(BONUS & SHIELD)
	{
		*adjShield = shield.strength + 0.02f;
	}
	if(BONUS & LIFE)
	{
		*adjLives = lives + 1;
	}
	
}

void Player::ResetTimer( Player::TimerType tType )
{
	switch(tType)
	{
	case FIRE_TIMER:
		fireT.StopWatch();
 		fireT.StartWatch();
		break;
	case MOVE_TIMER:
		moveT.StopWatch();
		moveT.StartWatch();
		break;
	case RECHARGE_TIMER:
		recharge.StopWatch();
		recharge.StartWatch();
		break;
	}
}

float Player::GetTime( Player::TimerType tType)
{	
	float temp = 0;
	switch(tType)
	{
	case FIRE_TIMER:
 		temp = fireT.GetTimeMilli();
		break;

	case MOVE_TIMER:
		temp  = moveT.GetTimeMilli();
		break;

	case RECHARGE_TIMER:
		temp = recharge.GetTimeMilli();
		break;
	}
	return temp;
}

Player::~Player()
{

}

//  End of Player Ship Class

Enemy::Enemy(int scrnWidth, int scrnHeight)
	: Ship( scrnWidth, scrnHeight),
	Nextp( NULL ),
	CurPoint( 1 )
{	
	Speed = 180;
	shieldRad = 0;
	shipSize = 32;
	shipHealth = 0.50f;
	shipValue = 10;
	ammoDamage = .10f;
	shipDamage = 0.20f;
	shield.strength = 0.0f;
	shield.rechargeRate = 0.0f;
	shield.respawnMax = 0.0f;
	startMoveTime = 0;
	wasHitByAmmo = false;
	state = Moving;


}

void Enemy::CalculatePath(Ship::Vertices* ePos, const Ship::Vertices* pPos)
{	
	rise = CalcRise( ePos->vy, pPos->vy );
	run = CalcRun( ePos->vx, pPos->vx );
	slope = CalcSlope( rise, run );
	float nrmlSpeed = GetSpeed(GetTime(MOVE_TIMER));
	nrmlSpeed = min(nrmlSpeed, 3);
	if(rise > run)
	{
		if(ePos->vx < pPos->vx)
		{
			ePos->vx += slope * nrmlSpeed;
		}
		else if(ePos->vx > pPos->vx)
		{
			ePos->vx -= slope * nrmlSpeed;
		}
		if(ePos->vy < pPos->vy)
		{
			ePos->vy += nrmlSpeed;
		}
		else if(ePos->vy > pPos->vy)
		{
			ePos->vy -= nrmlSpeed;
		}
	}
	else if(rise < run)
	{
		if(ePos->vx < pPos->vx)
		{
			ePos->vx += nrmlSpeed;
		}
		else if(ePos->vx > pPos->vx)
		{
			ePos->vx -= nrmlSpeed;
		}
		if(ePos->vy < pPos->vy)
		{
			ePos->vy += slope * nrmlSpeed;
		}
		else if(ePos->vy > pPos->vy)
		{
			ePos->vy -= slope * nrmlSpeed;
		}
	}
	else
	{
		if( ePos->vx < pPos->vx )
		{
			ePos->vx += nrmlSpeed;
		}
		else if( ePos->vx > pPos->vx)
		{
			ePos->vx -= nrmlSpeed;
		}
		if( ePos->vy < pPos->vy )
		{
			ePos->vy += nrmlSpeed;
		}
		else if( ePos->vy > pPos->vy )
		{
			ePos->vy -= nrmlSpeed;
		}
	}	
}

void Enemy::CheckforWayPoint( )
{
	dist = sqrt( ( rise * rise ) + ( run* run ) );
	if( dist < 15 )
	{
		if( CurPoint < NPoints - 1 )
		{		
			CurPoint++;
			Nextp = &Points[ CurPoint ];				
		}		
		else
		{
			shipHealth = 0;					
		}
	}	

}

void Enemy::Move()
{	
	Vertices ePos = {shipCenterX, shipCenterY};
	
	CalculatePath(&ePos, Nextp);
	SetPosition(ePos.vx, ePos.vy);
}

void Enemy::Fire()
{
	Projectile ammo(shipCenterX, shipCenterY);
	ammoVector.push_back(ammo);
}

void Enemy::Die()
{
}

void Enemy::DoState()
{	
	switch(state)
	{
		case Moving:
			Move();
			CheckforWayPoint();
		break;
	}
}

float Enemy::GetTime( Player::TimerType tType)
{	
	float temp = 0;
	switch(tType)
	{
	case FIRE_TIMER:
		stopFireTime = shipTimer.GetTimeMilli();
		temp = stopFireTime - startFireTime;
		startFireTime = stopFireTime;
		break;
	case MOVE_TIMER:
		stopMoveTime = shipTimer.GetTimeMilli();
		temp = stopMoveTime - startMoveTime;
		startMoveTime = stopMoveTime;
		break;
	}
	return temp;
}

void Enemy::SetTriggerCoord( unsigned int X, unsigned int Y )
{
	trigger.vx = X;
	trigger.vy = Y;
}

void Enemy::SetSpawnPoint( unsigned int X, unsigned int Y )
{
	SetTriggerCoord( X, Y );
	for(unsigned int i = 0; i < NPoints; i++)
	{		
		Points[ i ].vy += Y;
	}
	SetPosition( X, Y );
}

void Enemy::AdjustWaypoints( float offSet )
{	
	for(unsigned int i = 1; i < NPoints; i++)
	{		
		Points[ i ].vy -= offSet;
	}
}

Enemy::~Enemy()
{
}
// End of Enemy Base Class

Enemy1::Enemy1(int scrnWidth, int scrnHeight)
	: Enemy(scrnWidth, scrnHeight)
{
	NPoints = 3;
	Points = new Vertices[ NPoints ];
	Points[ 0 ].vx = 0;
	Points[ 0 ].vy = 0;
	Points[ 1 ].vx = scrnWidth;
	Points[ 1 ].vy = 0;
	Points[ 2 ].vx = scrnWidth / 2;
	Points[ 2 ].vy = scrnHeight + shipSize;
	Nextp = &Points[ 1 ];

	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 0;
	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 0;
}

Enemy2::Enemy2(int scrnWidth, int scrnHeight)
	: Enemy(scrnWidth, scrnHeight)
{
	NPoints = 3;
	Points = new Vertices[ NPoints ];
	shipCenterX = 0;
	shipCenterY = 0;
	Points[ 1 ].vx = scrnWidth / 2;
	Points[ 1 ].vy = scrnHeight;
	Points[ 2 ].vx = scrnWidth + shipSize;
	Points[ 2 ].vy = 0 - shipSize;
	Nextp = &Points[ 1 ];

	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -15;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 15;
	verts[1].vy = -15;
	// Point 3
	verts[2].vx = 15;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = -15;
	verts[3].vy = 15;
}

Enemy3::Enemy3(int scrnWidth, int scrnHeight)
	: Enemy(scrnWidth, scrnHeight)
{
	NPoints = 2;
	Points = new Vertices[ NPoints ];
	shipCenterX = 0;
	shipCenterY = 0;
	Points[ 1 ].vx = scrnWidth + shipSize;
	Points[ 1 ].vy = scrnHeight + shipSize;
	Nextp = &Points[ 1 ];

	numVerts = 8;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -7;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 7;
	verts[1].vy = -15;
	// Point 3
	verts[2].vx = 16;
	verts[2].vy = -7;
	// Point 4
	verts[3].vx = 16;
	verts[3].vy = 7;
	// Point 5
	verts[4].vx = 7;
	verts[4].vy = 16;
	//Point 6
	verts[5].vx = -7;
	verts[5].vy = 16;
	//Point 7
	verts[6].vx = -15;
	verts[6].vy = 7;
	//Point 8
	verts[7].vx = -15;
	verts[7].vy = -7;
}

Enemy4::Enemy4(int scrnWidth, int scrnHeight)
	: Enemy(scrnWidth, scrnHeight)
{
	NPoints = 7;
	Points = new Vertices[ NPoints ];
	Points[ 1 ].vx = ( scrnWidth / 2 ) + 50;
	Points[ 1 ].vy = 100;
	Points[ 2 ].vx = ( scrnWidth / 2 ) - 50;
	Points[ 2 ].vy = Points[ 1 ].vy + 100;
	Points[ 3 ].vx = Points[ 1 ].vx;
	Points[ 3 ].vy = Points[ 2 ].vy + 100;
	Points[ 4 ].vx = Points[ 2 ].vx;
	Points[ 4 ].vy = Points[ 3 ].vy + 100;
	Points[ 5 ].vx = Points[ 1 ].vx;
	Points[ 5 ].vy = Points[ 3 ].vy + 100;
	Points[ 6 ].vx = Points[ 2 ].vx;
	Points[ 6 ].vy = Points[ 4 ].vy + 100 + shipSize;
	Nextp = &Points[ 1 ];

	numVerts = 7;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -5;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = 0;
	verts[1].vy = -5;
	// Point 3
	verts[2].vx = -11;
	verts[2].vy = -5;
	// Point 4
	verts[3].vx = 0;
	verts[3].vy = 16;
	// Point 5
	verts[4].vx = 11;
	verts[4].vy = -5;
	// Point 6
	verts[5] = verts[1];
	// Point 7
	verts[6].vx = 5;
	verts[6].vy = -15;
}

Enemy5::Enemy5(int scrnWidth, int scrnHeight)
	: Enemy(scrnWidth, scrnHeight)
{
	NPoints = 5;
	Points = new Vertices[ NPoints ];
	Points[ 1 ].vx = 0;
	Points[ 1 ].vy = scrnHeight / 2;
	Points[ 2 ].vx = scrnWidth / 2;
	Points[ 2 ].vy = scrnHeight;
	Points[ 3 ].vx = scrnWidth;
	Points[ 3 ].vy = scrnHeight / 2;
	Points[ 4 ].vx = scrnWidth / 2 + shipSize;
	Points[ 4 ].vy = 0 - shipSize;
	Nextp = &Points[ 1 ];

	numVerts = 10;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = -9;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -6;
	verts[1].vy = -3;
	// Point 3
	verts[2].vx = -15;
	verts[2].vy = 4;
	// Point 4
	verts[3].vx = -3;
	verts[3].vy = 4;
	// Point 5
	verts[4].vx = 0;
	verts[4].vy = 16;
	// Point 6
	verts[5].vx = 4;
	verts[5].vy = 4;
	// Point 7
	verts[6].vx = 16;
	verts[6].vy = 4;
	// Point 8
	verts[7].vx = 7;
	verts[7].vy = -3;
	// Point 9
	verts[8].vx = 10;
	verts[8].vy = -15;
	// Point 10
	verts[9].vx = 0;
	verts[9].vy = -8;
}
// End of Enemy Derived classes

Boss1::Boss1(int scrnWidth, int scrnHeight)
	: Ship( scrnWidth, scrnHeight ),
	NMissiles( 10 ),
	BState( Attack ),
	FireRate(0.5f),
	RegenRate(3.0f),
	Missiles( new Missile[ NMissiles ]),
	LaunchTimer(0),
	RegenTimer(0),
	mCount(NMissiles)
{		
	shipHealth = 10;
	shield.strength = 1;
	shipSize = 32;
	shieldRad = 40;
	shipValue = 1000;
	numVerts = 4;
	verts = new Vertices[numVerts];
	// Point 1
	verts[0].vx = 0;
	verts[0].vy = -15;
	// Point 2
	verts[1].vx = -10;
	verts[1].vy = 0;
	// Point 3
	verts[2].vx = 0;
	verts[2].vy = 15;
	// Point 4
	verts[3].vx = 10;
	verts[3].vy = 0;

}

void Boss1::UpdateTimer()
{
	bossTimer.StopWatch();
	temp = bossTimer.GetTimeMilli();
	bossTimer.StartWatch();
}

void Boss1::DecideState()
{
	UpdateTimer();
	readyToFire = false;
	if(mCount < NMissiles )
	{
		BState = Attack;

		if(LaunchTimer < FireRate * 1000)
		{
			BState = Idle;
		}
	}
	else
	{
		if(RegenTimer < RegenRate * 1000)
		{
			BState = RegenMissiles;
		}
		else
		{
			mCount %= NMissiles;
		}
	}
}

void Boss1::DoState( int X, int Y)
{
	DecideState();
	
	switch( BState )
	{
	case Attack:
		readyToFire = true;
		LaunchTimer = 0;		
		mCount++;
		RegenTimer = 0;
		break;
	case RegenMissiles:	
		RegenTimer += temp;		
		break;
	case Idle:
		LaunchTimer += temp;
		break;
	}

	Missiles[ mCount ].DoState( X, Y, readyToFire);
}

const Ship::Vertices Boss1::GetMissile(unsigned int i)
{
	Vertices m = {Missiles[ i ].GetX(), Missiles[ i ].GetY()};
	return m;
}

bool Boss1::MissileExploded(unsigned int i)
{
	return Missiles[ i ].HasExploded();
}

const unsigned int Boss1::GetNMissiles()
{
	return NMissiles;
}

const int Boss1::GetMissileRad()
{
	return Missiles->GetMIR();
}

const float Boss1::GetMissileDamage()
{
	return Missiles->GetDamage();
}


