#pragma once
#include "Projectiles.h"
#include <vector>
#include "Timer.h"
#include <ctime>

class Ship
{
public:
	Ship(int scrnWidth, int scrnHeight);
	
	virtual ~Ship();

	enum ShipState
	{
		Moving,
		Firing,
		Dead
	};
	
	enum TimerType
	{
		FIRE_TIMER,
		MOVE_TIMER,
		RECHARGE_TIMER
	};

	enum DamageType
	{
		SHIP_DAMAGE, AMMO_DAMAGE
	};
	
	enum BonusFlags
	{
		NONE			= 0x0000,
		SPEED			= 0x000f,
		SHIELD			= 0x00f0,
		LIFE			= 0x0f00,
		BACKUP			= 0xf000
	};
	
	// Structure to hold the X and Y positions of the vertices of each ship
	// Can use DrawLine in the D3DGraphics class to connect each vertex
	struct Vertices
	{
		float vx, vy;
	};
	
	struct Shield
	{
		float strength, rechargeRate, respawnMax;
	};

	unsigned int GetNumVerts()const;
	float GetSpeed(float input);
	float GetShipCenterX();
	float GetShipCenterY();
	unsigned int GetShipSize();
	Vertices* GetVertList();
	unsigned int GetShieldRad();
	float GetShieldStrength();
	float GetDamage(DamageType damage);
	unsigned int GetShipValue();

	unsigned int GetAmmoListSize();
	void UpdateAmmo(unsigned int ammoListSize);
	void KillAmmo(unsigned int ammoIndex);
	Vertices GetAmmoPos(unsigned int ammoIndex);

	void SetShieldStrength( float damage );

	float GetHealth();
	void SetHealth( float health );
	void SetHitByAmmo( bool wasHit );
	bool GetHitByAmmo();

	// Helper functions
	bool CheckIsAlive( );
	float CalcRun( float x, float x2 );
	float CalcRise( float y, float y2 );
	float CalcSlope( float rise, float run );
	void SetClip( unsigned int x, unsigned y, unsigned int x2, unsigned int y2 );
	void SetPosition( unsigned int x, unsigned int y );	
	void SetScreenRes( unsigned int x, unsigned int y);

protected:
	unsigned int numVerts, shieldRad, shipSize, scrnWidth, scrnHeight, shipValue, camx, camy;
	float shipCenterX, shipCenterY, Speed, startFireTime, stopFireTime, startMoveTime,
		  stopMoveTime, shipHealth, ammoDamage, shipDamage, astroDamage;
	bool wasHitByAmmo;
	Shield shield;
	Vertices* verts;	
	std::vector<Projectile> ammoVector;
	Timer shipTimer;

};

class Player: public Ship
{
public:
	Player(int screnWidth, int scrnHeight);
	~Player();
	void DoState( float X, float Y, bool isShooting);
	void AddBonus(float* adjSpeed, float* adjShield, int* adjLives);
	int GetLives();

private:
	void Move( float scrnX, float scrnY);
	void Fire();
	void Die();
	void RechargeShield();
	void DecideState( bool isShooting );
	bool CheckReadyToFire();
	float GetTime( TimerType tType );
	void ResetTimer( TimerType tType );
	void Clip(int left, int top, int right, int bottom );

private:
	int lives;
	BonusFlags BONUS;
	ShipState state;
	Timer recharge, moveT, fireT;
	
};

class Enemy: public Ship
{
public:
	Enemy(int screnWidth, int scrnHeight);
	virtual ~Enemy();
	void Move();
	void Fire();
	void Die();
	void DoState();
	float GetTime( TimerType tType );
	void SetTriggerCoord( unsigned int X, unsigned int Y );
	void SetSpawnPoint( unsigned int X, unsigned int Y );
	void AdjustWaypoints( float offSet );
	void CheckforWayPoint();
	void CalculatePath(Ship::Vertices* ePos, const Ship::Vertices* pPos);
	
protected:
	unsigned int NPoints;
	unsigned int CurPoint;
	float rise, run, slope, dist;
	Vertices* Points;
	Vertices* Nextp;
	ShipState state;
	Vertices trigger;
};

class Enemy1: public Enemy
{
public:
	Enemy1(int scrnWidth, int scrnHeight);
};

class Enemy2: public Enemy
{
public:
	Enemy2(int scrnWidth, int scrnHeight);
};

class Enemy3: public Enemy
{
public:
	Enemy3(int scrnWidth, int scrnHeight);
};

class Enemy4: public Enemy
{
public:
	Enemy4(int scrnWidth, int scrnHeight);
};

class Enemy5: public Enemy
{
public:
	Enemy5(int scrnWidth, int scrnHeight);
};

class Boss1: public Ship
{
public:
	enum State
	{
		Attack,
		RegenMissiles,
		Idle
	};
	Boss1(int scrnWidth, int scrnHeight);
	void DoState( int X,  int Y);
	const Vertices GetMissile( unsigned int i );
	const int GetMissileRad();
	const unsigned int GetNMissiles();
	bool MissileExploded( unsigned int i );
	const float GetMissileDamage();
		
private:
	void DecideState();
	void MissilesRegen();
	void UpdateTimer();

	Timer bossTimer;
	float temp;						// stores progressive time between calls for event handling
		
	float LaunchTimer, RegenTimer;	// Event timer triggers
	
	State BState;
	int pX, pY;						// playerX and playerY to pass to Missile class
	const float FireRate, RegenRate;
	unsigned int NMissiles;			// Number Of Missiles
	unsigned int MissilesFired;		// Counter for firing missiles and event handling
	Missile* Missiles;
	unsigned int mCount;
	bool readyToFire;
	
};
